@@ -6,7 +6,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 use File::Copy;
 use File::Path;
@@ -81,7 +81,7 @@ sysrun untar_in pick ask
 hammer say
 sudo_me bin_find
 fs_read_open fs_write_open pipe_copy
-snip
+snip password_read nice_time
 );
 
 our %EXPORTABLE = map { $_ => 1 } @EXPORTABLE;
@@ -942,7 +942,37 @@ sub qquote {
 
 =item C<$quoted_string = quote($string, [$metachars])>
 
-Similar to C<qquote()>, just puts a string in single quotes.
+Similar to C<qquote()>, just puts a string in single quotes and
+escapes what needs to be escaped.
+
+Note that shells typically don't support escaped single quotes within
+single quotes, which means that
+
+    $ echo 'foo\'bar'
+    >
+
+is invalid and the shell waits until it finds a closing quote. 
+Instead, there is an evil trick which gives the desired result:
+
+    $ echo 'foo'\''bar'  # foo, single quote, \, 2 x single quote, bar
+    foo'bar
+
+It uses the fact that shells interpret back-to-back strings as one.
+The construct above consists of three back-to-back strings: 
+
+    (1) 'foo'
+    (2) '
+    (3) 'bar'
+
+which all get concatenated to a single 
+
+    foo'bar
+
+If you call C<quote()> with C<$metachars> set to ":shell", it will
+perform that magic behind the scenes:
+
+    print quote("foo'bar");
+      # prints: 'foo'\''bar'
 
 =cut
 
@@ -951,10 +981,14 @@ sub quote {
 ###############################################
     my($str, $metas) = @_;
 
-    $str =~ s/([\\'])/\\$1/g;
+    if(defined $metas and $metas eq ":shell") {
+        $str =~ s/([\\])/\\$1/g;
+        $str =~ s/(['])/'\\''/g;
+    } else {
+        $str =~ s/([\\'])/\\$1/g;
+    }
 
-    if(defined $metas) {
-        $metas = '' if $metas eq ":shell";
+    if(defined $metas and $metas ne ":shell") {
         $metas =~ s/\]/\\]/g;
         $str =~ s/([$metas])/\\$1/g;
     }
@@ -1332,6 +1366,72 @@ sub printable {
 
 =pod
 
+=item C<read_password($prompt)>
+
+Reads in a password to be typed in by the user in noecho mode.
+A call to read_password("password: ") results in
+
+    password: ***** (stars aren't actually displayed)
+
+This function will switch the terminal back into normal mode
+after the user hits the 'Return' key.
+
+=cut
+
+###########################################
+sub password_read {
+###########################################
+    my($prompt) = @_;
+
+    use Term::ReadKey;
+    ReadMode 'noecho';
+    $| = 1;
+    print "$prompt";
+    my $pw = ReadLine 0;
+    chomp $pw;
+    ReadMode 'restore';
+    print "\n";
+
+    return $pw;
+}
+
+=pod
+
+=item C<nice_time($time)>
+
+Format the time in a human-readable way, less wasteful than the 
+'scalar localtime' formatting. 
+
+    print nice_time(), "\n";
+      # 2007/04/01 10:51:24
+
+It uses the system time by default, but it can also accept epoch seconds:
+
+    print nice_time(1170000000), "\n";
+      # 2007/01/28 08:00:00
+
+It uses localtime() under the hood, so the outcome of the above will
+depend on your local time zone setting.
+
+=cut
+
+###########################################
+sub nice_time {
+###########################################
+    my($time) = @_;
+
+    $time = time() unless defined $time;
+
+    my ($sec,$min,$hour,$mday,$mon,$year,
+     $wday,$yday,$isdst) = localtime($time);
+
+    return sprintf("%d/%02d/%02d %02d:%02d:%02d",
+     $year+1900, $mon+1, $mday,
+     $hour, $min, $sec);
+}
+
+=pod
+
 =back
 
 =head1 AUTHOR
@@ -1340,7 +1440,7 @@ Mike Schilli, E<lt>m@perlmeister.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2004 by Mike Schilli
+Copyright (C) 2004-2007 by Mike Schilli
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.3 or,
